//
// GameX -- CameraX Demo
// 
// Copyright (C) 2002 Rama C. Hoetzlein
// 
// This software is released under the GameX GNU GPL 
// Open Source Liscense. See the GameX documentation included
// with this source code for terms of modification, 
// distribution and re-release. 
// 

#define GAMEX_MAIN

#include "gamex.hpp"
										//*** Global Variables & Defines
#define MAX_DOTS 1000					// Maximum number of "Dots" we think we'll need
int num_dots = 0;						// Total number of Dots currently used
Vector3DF dot_list[MAX_DOTS];			// Array of cube dot positions
CameraX cam;							// Camera for 3D projections
ImageX dot_img;							// Image of a single "Dot"

void drawCube ()						//*** Draw the 3D Cube of Dots
{
	for (int n = 0; n < num_dots; n++) {
		GameX.SetDrawScale (2.0f); // make each dot 2 units across
		GameX.DrawImage3D (&dot_img, dot_list[n]); // project, sort, and draw the dot
	}
}

void GameInit (void)
{											// Initialize GameX
	GameX.Initialize ("CamDemoX", VIDEO_16BIT, 800, 600);
	
	cam.SetWindow (0, 0, GameX.GetWidth(), GameX.GetHeight());	// Initialize Camera
	cam.SetToPosition (Vector3DF(0, 0, 0)); // Set camera to look at the origin
	GameX.SetDrawCam (&cam); // Tell GameX to draw using the camera
	
	// Create the image of a Dot:
	dot_img.Create (128, 128, true); // 128x128 pixels, with transparency
	for (int px = 0 ; px < 128 ; px++)
		for (int py = 0 ; py < 128 ; py++)
			if (64.0f > sqrtf ((float) ((px-64)*(px-64)+(py-64)*(py-64))))
				dot_img.SetPixel (px,py, px,py,px+py,192+py-px);

	// Initialize Array of Dots in Cube:
	for (int x = -12 ; x <= 12 ; x += 4)
		for (int y = -12 ; y <= 12 ; y += 4)
			for (int z = -12 ; z <= 12 ; z += 4)
				if (num_dots < MAX_DOTS)
					dot_list[num_dots++].Set((float)x, (float)y, (float)z); // Set each dot's position
}

void GameRun (void)
{	
	// Calculate the camera's angles and distance from where it's looking at:
	float x_angle = (float) GameX.GetGlobalCounter ();
	float y_angle = GameX.GetOscillatingValue (0.0f, 60.0f, 1.0f);
	float z_dist  = GameX.GetOscillatingValue (50.0f, 100.0f, 0.5f);
	cam.SetFromAngles (Vector3DF(x_angle, y_angle, z_dist));
}

void drawAxis ()						//*** Draw lines to show XYZ
{
	GameX.SetDrawScale (0.33f);				// Draw red line on X axis:
	GameX.DrawLine3D (ColorX(255,0,0), Vector3DF(0,0,0), Vector3DF(24,0,0));
	GameX.SetDrawScale (0.33f);				// Draw green line on Y axis:
	GameX.DrawLine3D (ColorX(0,255,0), Vector3DF(0,0,0), Vector3DF(0,24,0));
	GameX.SetDrawScale (0.33f);				// Draw blue line on Z axis:
	GameX.DrawLine3D (ColorX(0,0,255), Vector3DF(0,0,0), Vector3DF(0,0,24));
}

void GameDraw (void)
{
	GameX.ClearScreen();
	GameX.Begin3DScene ();
	drawCube ();
	drawAxis ();
	GameX.End3DScene ();
}

