//
// GameX
// Ship Demo - Photon Class Header
//
// Copyright (C) 2002 Rama C. Hoetzlein
// 
// This software is released under the GameX GNU GPL 
// Open Source Liscense. See the GameX documentation included
// with this source code for terms of modification, 
// distribution and re-release. 
// 


#ifndef PHOTONS_DEFINED
	#define PHOTONS_DEFINED

	#include "gamex.hpp"
	
	// Photons are implemented as a simple 2-dimensional particle system.

	#define MAX_AGE 48					// Photon constant, maximum age of photons
	#define MAX_PHOTONS 1024			// Photon constant, maximum photons active at once

	struct Photon {						//*** Photon structure
		float		x, y;				// Position of photon (2D) in game screen coordinates
		float		dx, dy;				// Velocity of photon (2D)
		int			age;				// Age of photon, 0 meaning new
		int			img;				// Image number of photon (only image # 0 is used for now)
	};

	class Photons {						//*** Photon class
	public:
		void Initialize (void);			// Initialize system of photons (i.e. load the necessary image(s))
		void Update (void);				// Move all photons
		void Draw (void);				// Draw all photons
										// Add a photon to the system:
		int AddPhoton (float new_x, float new_y, float new_dx, float new_dy);
	private:
		int		num_photons;				// Current number of photons used out of MAX_PHOTONS
		Photon	photon_list[MAX_PHOTONS];	// Array of photons (ie. 'the particle system')
		ImageX	photon_img[30];				// Photon images (only 1st image currently used)
	};

#endif