//
// GameX
// Ship Demo - Ship Class Header
// 
// Copyright (C) 2002 Rama C. Hoetzlein
// 
// This software is released under the GameX GNU GPL 
// Open Source Liscense. See the GameX documentation included
// with this source code for terms of modification, 
// distribution and re-release. 
// 

#ifndef SHIP_DEFINED
	#define SHIP_DEFINED

	#include "gamex.hpp"
	#include "shipdemox-photons.hpp"

	// The Ships are implemented as simple Finite-State-Machines
											
	#define SHIP_IMG_LEFT			0		// Ship Frame Sprite Numbers:
	#define SHIP_IMG_FLAT			9
	#define SHIP_IMG_RIGHT			17

	class Ship {							//*** Ship class
	public:
		void Initialize (int num);			// Initialize ship of number 'num'
		void Update (void);					// Get user input and move the ship
		void Draw (void);					// Draw the ship
		
		void CollideCheck (Ship* ship2);	// Collision Detection with another ship

		float GetX (void) {return x;}		// Get position and velocity of ship
		float GetY (void) {return y;}
		float GetVX (void) {return vx;}
		float GetVY (void) {return vy;}
	private:
		int			ship_num;				// Current ship number
		
		float		ang, dang;				// Angle of ship (and change in angle)
		float		x, y;					// Position of ship
		float		vx, vy;					// Velocity of ship
		int			refire;					// Refire counter for ship

		int			curr_img;				// Current ship image
		float		target_x, target_y;		// Offset of ship's target
		
		ImageX		ships;					// Ship images of this ship
		ImageX		target;					// Ship's Target image

		SoundX		photon_snd;				// Sound of photon firing
	};

#endif