#include "sprite.hpp"

Sprite::Sprite(int kind, int trail_length, Vector3DF initialPoint)
{
	type = kind;
	trailLength = max(1,trail_length);
	position = initialPoint;
	velocity = Vector3DF(0,0,0);
	current = 0;
	for(int i = 0 ; i < trailLength ; i++)
		trailPosition[i] = initialPoint;
	curr_img = 0;
	damage = 0;

	// now do any sprite-type-specific initialization:
	switch(kind) {
		case SPRITE_MAIN_CHAR:
			curr_img = 9;
			break;
	}
}

Sprite::~Sprite(void)
{
}

void Sprite::update(void)
{
	switch(type) {
		case SPRITE_MAIN_CHAR: {
			float xMove;
			int iLast = trailLength-2;
			int lastIndex = (current + iLast + 1) % trailLength;
			int i = trailLength-1;
			int index = (current + i + 1) % trailLength;
			Vector3DF v = trailPosition[index]-trailPosition[lastIndex];
			xMove = v.x - velocity.x;

			if(xMove > 0.499f) xMove = 0.499f;
			if(xMove < -0.5f) xMove = -0.5f;
			int new_img = (int)((xMove+0.5f)*18.0f);
			if(new_img > curr_img)
				curr_img++;
			else if(new_img < curr_img)
				curr_img--;

			if(position.x >= 400 && velocity.x >= 0
			|| position.x <= -400 && velocity.x <= 0) {
				velocity.x *= 1.0f/((fabsf(position.x)+1.0f)/400.0f);
				if(position.x > 600) position.x = 600;
				if(position.x < -600) position.x = -600;
			}

			if(damage >= 10000) {
				float osc = 3.5f;
				trailVelocity[current] = Vector3DF(GameX.GetRandomFloat(-osc,osc), GameX.GetRandomFloat(-osc,osc), GameX.GetRandomFloat(-osc,osc));
			}

		}	break;
	}

	++current %= trailLength;

	position += velocity;


	switch(type) {
	default:
	case SPRITE_MAIN_CHAR:
		trailPosition[current] = position;
 		trailVelocity[current] = Vector3DF(0,0,0);
		break;
	case SPRITE_FIREBALL:
		trailPosition[current] = position;
		trailVelocity[current] = Vector3DF(GameX.GetRandomFloat(-2.0f,2.0f), GameX.GetRandomFloat(-2.0f,0.0f), GameX.GetRandomFloat(-2.0f,2.0f));
		break;
	case SPRITE_ICEBALL:
		trailPosition[current] = position;
		trailVelocity[current] = Vector3DF(GameX.GetRandomFloat(-2.0f,2.0f), GameX.GetRandomFloat(0.0f,2.0f), GameX.GetRandomFloat(-2.0f,2.0f));
		break;
	}

	for(int i = 0 ; i < trailLength ; i++) {
		trailPosition[i] += trailVelocity[i];
	}
}

void Sprite::draw(void)
{
	// draw the sprite's trail:
	int lastIndex = -1;
	for(int i = 0 ; i < trailLength ; i++) {
		int index = (current + i + 1) % trailLength;
		float bright = (float)(i+1) / (float)trailLength;
		int c = (int)(256.0f*bright);

		switch(type) {
		case SPRITE_MAIN_CHAR:
			{
					if(lastIndex != -1) {
						Vector3DF v = trailPosition[index]-trailPosition[lastIndex];
						v *= 1.2f * 1.414f;

						GameX.SetDrawScale(bright*16.0f * 1.414f);
						GameX.SetDrawShading(ColorX(c/4,c/2,c));
						GameX.SetDrawMode(DRAW_ADD);
						GameX.DrawLineImage3D(&rp.jetImage[(i+GameX.GetGlobalCounter()/3)%24],trailPosition[lastIndex]-v, trailPosition[index]+v, true);

						if(damage >= 10000) {
		
							// rainbow-colored explosion colors based on velocity components
							int r = (int)GameX.GetOscillatingValue(0,32,trailVelocity[lastIndex].x);
							int g = (int)GameX.GetOscillatingValue(0,64,trailVelocity[lastIndex].y);
							int b =(int) GameX.GetOscillatingValue(0,128,trailVelocity[lastIndex].z);

							GameX.SetDrawScale(bright);
							GameX.SetDrawShading(ColorX(r,g,b));
							GameX.SetDrawMode(DRAW_ADD|DRAWOP_DITHER);
							GameX.DrawLineImage3D(&rp.fireImage,trailPosition[lastIndex]-v, trailPosition[index]+v, false);
						}
					}
			}
			break;
		case SPRITE_FIREBALL:
			GameX.SetDrawScale(bright*64.0f);
			GameX.SetDrawMode(DRAW_ADD|DRAWOP_CENTERED);
			GameX.SetDrawAngle((float)(GameX.GetGlobalCounter()*2+i*30));
			GameX.SetDrawShading(ColorX(c/2,c/4,c/8));
			GameX.DrawImage3D(&rp.fireImage,trailPosition[index]);
			break;
		case SPRITE_ICEBALL:
			GameX.SetDrawScale(bright*64.0f);
			GameX.SetDrawMode(DRAW_ADD|DRAWOP_CENTERED);
			GameX.SetDrawAngle((float)(GameX.GetGlobalCounter()*2+i*30));
			GameX.SetDrawShading(ColorX(c/8,c/4,c/2));
			GameX.DrawImage3D(&rp.iceImage,trailPosition[index]);
			break;
		}
		lastIndex = index;
	}

	// draw the sprite itself, if it consists of something else than the trail:
	switch(type) {
		case SPRITE_MAIN_CHAR: { // the main character (space ship)
			if(damage >= 10000) break;

			// set what part of the ship "sprite sheet" to draw based on curr_img
			int left = (curr_img%5)*81;
			int top = (curr_img/5)*81;
			GameX.SetDrawPart(left,top,left+80,top+80);

			// calculate a unit vector v tangent to the ship's trajectory,
			// which we will use to orient the drawing of the ship along its path
			Vector3DF v = velocity;
			v.Normalize();

			// draw the actual ship image:
			// (we draw it as a "line sprite" here so we don't have to figure out the rotation angle on our own)
			GameX.SetDrawScale(64-velocity.Length());
			GameX.DrawLineImage3D(&rp.shipsImage,position - v*(128.0f*0.1f), position + v*(128.0f*0.9f), true);

			// draw the transparent shield in much the same way as we drew the ship,
			// except also offset toward camera a bit to make sure it gets drawn on top of the ship:
			GameX.SetDrawScale(64-velocity.Length());
			GameX.SetDrawMode(DRAW_GHOST); // set to "ghost" color-based transparency (a sort of auto-alpha layering)
			GameX.DrawLineImage3D(&rp.shellImage,position - v*(128.0f*0.1f)+ Vector3DF(0,0,-4), position + v*(128.0f*0.9f)+ Vector3DF(0,0,-4), true);

		}	break;
		case SPRITE_STAR: { // a star dot:
			GameX.SetDrawScale(4.0f);
			GameX.DrawPoint3D(ColorX(255,255,255),position);
			// note: DrawPoint functions simply draw a solid square dot,
			//       so we could probably make the stars look prettier by using DrawImage3D to draw star images instead
		}	break;
	}

}

void Sprite::accelerate(Vector3DF accelerationVector)
{
	velocity += accelerationVector;
}

void Sprite::attractTo(Vector3DF attractionPoint, float multiple)
{
	Vector3DF v = attractionPoint - position;
	v.Normalize();
	velocity += v*multiple;
}

void Sprite::enforceMaxDistance(Vector3DF centerPoint, float maxDist, float pullMultiple)
{
	if(centerPoint.Dist(position) > maxDist) {
		float distTotal = 0;
		for(int i = 0 ; i < trailLength ; i++) {
			int index = (current + i + 1) % trailLength;
			distTotal += centerPoint.Dist(trailPosition[index]);
		}
		distTotal /= trailLength;
		
		if(distTotal > maxDist) {
			position -= centerPoint;
			position *= pullMultiple;
			position += centerPoint;
		}
	}
}

void Sprite::multiplyVelocity(float m)
{
	velocity *= m;
}

void Sprite::collideCheck(Sprite* s)
{
	Vector3DF vector = s->position - position; // Calculate vector to 2nd sprite

	float dist = vector.Length();

	if(vector.x || vector.y || vector.z)
		vector /= dist;  // normalize 
	else
		vector.y = 1.0f; // avoid possible trouble when sprites exactly overlap
	
	if (dist<48) {       // if distance to 2nd sprite is less than a certain number of pixels,
		float speed = velocity.Length();
		float speed2 = s->velocity.Length();
		// calculate 'rebound' vector to throw sprites in opposite directions
		// (with speed equal to the sum of speeds of the two sprites)
		float factor = 0.5f; // weight ratio
		float factor2 = 0.5f; // 0.5::0.5  --  sum (0.5+0.5) must = 1.0 for realistic collision

		if(s->type == SPRITE_MAIN_CHAR) {
			factor = 0.5f;
			factor2 = 1.0f; // not physically possible since sum = 1.5, but this gives extra effect to collision with ship
		}
		velocity -= vector * ((speed + speed2)/2 * factor);
		s->velocity += vector * ((speed + speed2)/2 * factor2);

		if(s->type == SPRITE_MAIN_CHAR) {
			s->damage += (int)(((vector * ((speed + speed2)/2 * factor)).LengthSq()) * 2.0f);

			if(s->damage >= 10000) {
				for(int i = 0 ; i < s->trailLength ; i++)
					s->trailVelocity[i] += Vector3DF(GameX.GetRandomFloat(-10.0f,10.0f),
					                                 GameX.GetRandomFloat(-20.0f,10.0f),
													 GameX.GetRandomFloat(-10.0f,10.0f));
			}
		}
	
	}

	if(s->type == SPRITE_MAIN_CHAR) {

		Vector3DF vector0 = s->position - position;
		Vector3DF vector1 = s->trailPosition[(s->current + 1 + s->trailLength) % s->trailLength] - position;

		bool ok = true;
		if((vector0.x < (-256.0f*0.707f)
		&& vector1.x < (-256.0f*0.707f))
		|| (vector0.x > (256.0f*0.707f)
		&& vector1.x > (256.0f*0.707f))
		|| (vector0.y < (-256.0f*0.707f)
		&& vector1.y < (-256.0f*0.707f))
		|| (vector0.y > (256.0f*0.707f)
		&& vector1.y > (256.0f*0.707f)))
			ok = false;
		
		if(ok)
		for(int i = 0 ; i < s->trailLength ; i++) {

			int index = (s->current + i + 1) % s->trailLength;

			Vector3DF vector = s->trailPosition[index] - position; // Calculate vector to 2nd sprite's trail

			if(vector.x < (32.0f*0.707f)
			&& vector.x > (-32.0f*0.707f)
			&& vector.y < (32.0f*0.707f)
			&& vector.y > (-32.0f*0.707f))
				continue;

			float dist = vector.Length();

			if(vector.x || vector.y || vector.z)
				vector /= dist;  // normalize 
			else
				vector.y = 1.0f; // avoid possible trouble when sprites exactly overlap

			Vector3DF vector2 = Vector3DF(0,-1,0);

			if (dist<32.0f) {       // if distance to 2nd sprite is less than a certain number of pixels,

				float speed = velocity.Length();
				float speed2 = s->trailVelocity[index].Length();
				// calculate 'rebound' vector to throw sprites in opposite directions
				// (with speed equal to the sum of speeds of the two sprites)
				velocity -= vector2 * ((speed + speed2)/2 * 0.45f); // (almost equal weight ratio) (note: this collision is weird)
				s->trailVelocity[index] += vector * ((speed + speed2)/2 * 0.5f);
			}
		}
	}
}

void Sprite::moveTo(Vector3DF newLocation)
{
	position = newLocation;
}
